import express,{Router, Request, Response } from 'express';
import {Task} from '../models/Task';
import {myDataSource} from '../app-data-source';
import { DateTimeTask } from '../models/DateTimeTask';
import { auth, authAuthorOrExecutorOfDateTimeTask, authAuthorOrExecutorOfTask,taskFinderById } from '../helpers';
const router:Router = express.Router();
const dataSource = myDataSource;


/** make copy of task in calendar view */
router.post("/make-copy",authAuthorOrExecutorOfTask, async(req:Request, res:Response):Promise<Response>=>{
    const {taskId, dateTimeDue, dateTimeStart} = req.body;
    const task = await taskFinderById(taskId);
    if(!task) return res.status(401).send({message:'task with possible user involved is not found'})
    const newDateTimeTask = new DateTimeTask();
    newDateTimeTask.dateTimeStart = dateTimeStart;
    newDateTimeTask.dateTimeDue = dateTimeDue;
    newDateTimeTask.task = taskId;
    await newDateTimeTask.save();
    return res.send({task})

} )

/** change date time of copy of task in calendar view */
router.put("/change-copy/:dateTimeTaskId", authAuthorOrExecutorOfTask, async(req:Request, res: Response):Promise<Response>=>{
    const {dateTimeTaskId} = req.params
    const {executorStatus, taskId, dateTimeStart, dateTimeDue, description, title, priority} = req.body
    const dateTimeTask = await dataSource
    .createQueryBuilder()
    .select('dateTimeTask')
    .from(DateTimeTask,'dateTimeTask')
    .where("dateTimeTask.id = :dateTimeTaskId",{dateTimeTaskId})
    .getOne()
    if(!dateTimeTask) return res.send({message:"such dateTimeTask does not exists"})
    dateTimeTask.dateTimeStart=dateTimeStart
    dateTimeTask.dateTimeDue=dateTimeDue
    await dateTimeTask.save()
    const task = await taskFinderById(taskId)
    if (!task) return res.status(404).send({Message:'task not found'})
    task.title = title; 
    task.description = description;
    task.priority = priority;
    await task.save()
    return res.send({task}) 
})

/**delete copyTask by dateTimeTaskId */
router.delete('/:dateTimeTaskId',authAuthorOrExecutorOfDateTimeTask, async(req:Request, res:Response):Promise<Response|void>=>{
    const {executorStatus} = req.body
    if(executorStatus){
    const {dateTimeTaskId} = req.params
    await myDataSource
    .createQueryBuilder()
    .delete()
    .from(DateTimeTask)
    .where("id = :dateTimeTaskId", { dateTimeTaskId })
    .execute()
    return res.send({message:"copyTask delete succesfully"})
}
}
)

export default router;
